from paracelsus.graph import get_graph_string


def test_get_graph_string(package_path):
    graph_string = get_graph_string("example.base:Base", ["example.models"], [package_path], "mermaid")

    assert "users {" in graph_string
    assert "posts {" in graph_string
    assert "comments {" in graph_string

    assert "users ||--o{ posts : author" in graph_string
    assert "posts ||--o{ comments : post" in graph_string
    assert "users ||--o{ comments : author" in graph_string

    assert "CHAR(32) author FK" in graph_string
    assert 'CHAR(32) post FK "nullable"' in graph_string
    assert "DATETIME created" in graph_string



import importlib
import os
import sys
from pathlib import Path
from typing import List

from .transformers.dot import Dot
from .transformers.mermaid import Mermaid

transformers = {
    "mmd": Mermaid,
    "mermaid": Mermaid,
    "dot": Dot,
    "gv": Dot,
}


def get_graph_string(
    base_class_path: str,
    import_module: List[str],
    python_dir: List[Path],
    format: str,
) -> str:
    # Update the PYTHON_PATH to allow more module imports.
    sys.path.append(str(os.getcwd()))
    for dir in python_dir:
        sys.path.append(str(dir))

    # Import the base class so the metadata class can be extracted from it.
    # The metadata class is passed to the transformer.
    module_path, class_name = base_class_path.split(":", 2)
    base_module = importlib.import_module(module_path)
    base_class = getattr(base_module, class_name)
    metadata = base_class.metadata

    # The modules holding the model classes have to be imported to get put in the metaclass model registry.
    # These modules aren't actually used in any way, so they are discarded.
    # They are also imported in scope of this function to prevent namespace pollution.
    for module in import_module:
        if ":*" in module:
            # Sure, execs are gross, but this is the only way to dynamically import wildcards.
            exec(f"from {module[:-2]} import *")
        else:
            importlib.import_module(module)

    # Grab a transformer.
    if format not in transformers:
        raise ValueError(f"Unknown Format: {format}")
    transformer = transformers[format]

    # Save the graph structure to string.
    return str(transformer(metadata))



import os
import tomllib
from pathlib import Path
from typing import Any, Dict


def get_pyproject_settings(dir: Path = Path(os.getcwd())) -> Dict[str, Any] | None:
    pyproject = dir / "pyproject.toml"

    if not pyproject.exists():
        return None

    with open(pyproject, "rb") as f:
        data = tomllib.load(f)

    return data.get("tool", {}).get("paracelsus", None)



from paracelsus.transformers.mermaid import Mermaid


def test_mermaid(metaclass):
    mermaid = Mermaid(metaclass=metaclass)
    graph_string = str(mermaid)

    assert "users {" in graph_string
    assert "posts {" in graph_string
    assert "comments {" in graph_string

    assert "users ||--o{ posts : author" in graph_string
    assert "posts ||--o{ comments : post" in graph_string
    assert "users ||--o{ comments : author" in graph_string

    assert "CHAR(32) author FK" in graph_string
    assert 'CHAR(32) post FK "nullable"' in graph_string
    assert 'BOOLEAN live "True if post is published,nullable"' in graph_string
    assert "DATETIME created" in graph_string



from typer.testing import CliRunner

from paracelsus.cli import app

runner = CliRunner()


def test_graph(package_path):
    result = runner.invoke(
        app, ["graph", "example.base:Base", "--import-module", "example.models", "--python-dir", str(package_path)]
    )

    assert result.exit_code == 0

    assert "users {" in result.stdout
    assert "posts {" in result.stdout
    assert "comments {" in result.stdout

    assert "users ||--o{ posts : author" in result.stdout
    assert "posts ||--o{ comments : post" in result.stdout
    assert "users ||--o{ comments : author" in result.stdout

    assert "CHAR(32) author FK" in result.stdout
    assert 'CHAR(32) post FK "nullable"' in result.stdout
    assert "DATETIME created" in result.stdout


def test_inject(package_path):
    result = runner.invoke(
        app,
        [
            "inject",
            str(package_path / "README.md"),
            "example.base:Base",
            "--import-module",
            "example.models",
            "--python-dir",
            str(package_path),
            "--check",
        ],
    )

    assert result.exit_code == 1


def test_version():
    result = runner.invoke(app, ["version"])

    assert result.exit_code == 0
